Exception: Blockenspiel::DSLMissingError

Inherits:
BlockenspielError show all
Defined in:
lib/blockenspiel/errors.rb

Overview

This exception is rasied when attempting to use the :proxy or :mixin parameterless behavior with a target that does not have the DSL module included. It is an error made by the DSL implementor.